"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.waitForPort = exports.waitForExpectedResult = exports.sleep = void 0;
const node_net_1 = require("net");
const node_assert_1 = require("assert");
const checkPort = (port, host) => new Promise((resolve) => {
    const client = (0, node_net_1.createConnection)(port, host);
    client.on('connect', () => {
        client.destroy();
        resolve(true);
    });
    client.on('error', () => {
        resolve(false);
    });
});
/** Wait until the defined milliseconds. */
const sleep = (milliseconds) => {
    /* c8 ignore next 3 */
    if (!Number.isInteger(milliseconds)) {
        throw new Error('Milliseconds must be an integer.');
    }
    return new Promise((resolve) => setTimeout(resolve, milliseconds));
};
exports.sleep = sleep;
/** Wait until a result is equal the expected value. */
const waitForExpectedResult = (callback, expectedResult, options) => __awaiter(void 0, void 0, void 0, function* () {
    const delay = (options === null || options === void 0 ? void 0 : options.delay) || 0;
    const interval = (options === null || options === void 0 ? void 0 : options.interval) || 100;
    const timeout = (options === null || options === void 0 ? void 0 : options.timeout) || 60000;
    /* c8 ignore start */
    if (typeof callback !== 'function') {
        throw new Error('Callback must be a function.');
    }
    if (!Number.isInteger(interval)) {
        throw new Error('Interval must be an integer.');
    }
    if (!Number.isInteger(timeout)) {
        throw new Error('Timeout must be an integer.');
    }
    if (!Number.isInteger(delay)) {
        throw new Error('Delay must be an integer.');
    }
    /* c8 ignore stop */
    yield (0, exports.sleep)(delay);
    const startTime = Date.now();
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const result = yield callback();
        if (typeof expectedResult === 'function') {
            if (typeof result === 'function' && result.name === expectedResult.name) {
                break;
            }
        }
        else if (typeof expectedResult === 'symbol') {
            if (typeof result === 'symbol' &&
                String(result) === String(expectedResult)) {
                break;
            }
        }
        else {
            try {
                (options === null || options === void 0 ? void 0 : options.strict)
                    ? (0, node_assert_1.deepStrictEqual)(result, expectedResult)
                    : (0, node_assert_1.deepEqual)(result, expectedResult);
                break;
            }
            catch (_a) { }
        }
        if (Date.now() - startTime >= timeout) {
            throw new Error('Timeout');
        }
        yield (0, exports.sleep)(interval);
    }
    yield (0, exports.sleep)(delay);
});
exports.waitForExpectedResult = waitForExpectedResult;
/* c8 ignore next 2 */ // ?
/** Wait until the defined port is active. */
const waitForPort = (port, options) => __awaiter(void 0, void 0, void 0, function* () {
    const host = (options === null || options === void 0 ? void 0 : options.host) || 'localhost';
    if (!Number.isInteger(port)) {
        throw new Error('Port must be an integer.');
    }
    yield (0, exports.waitForExpectedResult)(() => __awaiter(void 0, void 0, void 0, function* () { return yield checkPort(port, host); }), true, options);
});
exports.waitForPort = waitForPort;
